<?php

namespace Drupal\lupus_ce_renderer;

use Drupal\Core\Cache\CacheableJsonResponse;
use Drupal\Core\Controller\TitleResolverInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\PageCache\ResponsePolicy\KillSwitch;
use Drupal\Core\Render\MainContent\MainContentRendererInterface;
use Drupal\Core\Render\RenderContext;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\metatag\MetatagManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Block\BlockManager;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Core\Site\Settings;

/**
 * Renders main content only.
 */
class CustomElementsRenderer implements MainContentRendererInterface {

  /**
   * The title resolver.
   *
   * @var \Drupal\Core\Controller\TitleResolverInterface
   */
  protected $titleResolver;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The renderer configuration array.
   *
   * @var array
   * @see sites/default/default.services.yml
   */
  protected $rendererConfig;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The messenger interface..
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The block plugin manager.
   *
   * @var \Drupal\Core\Block\BlockManager
   */
  protected $blockManager;

  /**
   * The drupal settings.
   *
   * @var \Drupal\Core\Site\Settings
   */
  protected $settings;

  /**
   * The metatag manager.
   *
   * @var \Drupal\metatag\MetatagManagerInterface
   */
  protected $metatagManager;

  /**
   * The kill switch.
   *
   * @var \Drupal\Core\PageCache\ResponsePolicy\KillSwitch
   */
  protected $killSwitch;

  /**
   * Constructs the renderer.
   *
   * @param \Drupal\Core\Controller\TitleResolverInterface $title_resolver
   *   The title resolver.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer.
   * @param array $renderer_config
   *   The renderer configuration array.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger interface.
   * @param \Drupal\Core\Block\BlockManager $blockManager
   *   The block manager.
   * @param \Drupal\Core\Site\Settings $settings
   *   The drupal settings.
   * @param \Drupal\metatag\MetatagManagerInterface $metatagManager
   *   The module handler.
   * @param \Drupal\Core\PageCache\ResponsePolicy\KillSwitch $killSwitch
   *   The kill switch.
   */
  public function __construct(TitleResolverInterface $title_resolver, RendererInterface $renderer, array $renderer_config, ModuleHandlerInterface $module_handler, MessengerInterface $messenger, BlockManager $blockManager, Settings $settings, MetatagManagerInterface $metatagManager, KillSwitch $killSwitch) {
    $this->titleResolver = $title_resolver;
    $this->renderer = $renderer;
    $this->rendererConfig = $renderer_config;
    $this->moduleHandler = $module_handler;
    $this->messenger = $messenger;
    $this->blockManager = $blockManager;
    $this->settings = $settings;
    $this->metatagManager = $metatagManager;
    $this->killSwitch = $killSwitch;
  }

  /**
   * {@inheritdoc}
   */
  public function renderResponse(array $main_content, Request $request, RouteMatchInterface $route_match) {
    // First render the main content, because it might provide a title.
    $render_context = new RenderContext();
    $content = $this->renderer->executeInRenderContext($render_context, function () use (&$main_content) {
      return $this->renderer->render($main_content, TRUE);
    });
    // RendererInterface::render() always causes bubbleable metadata to be
    // stored in the render context, no need to check it conditionally.
    // We need it to forward caching information.
    // @see HtmlRenderer
    /** @var \Drupal\Core\Render\BubbleableMetadata $bubbleable_metadata */
    $bubbleable_metadata = !$render_context->isEmpty() ? $render_context->pop() : new BubbleableMetadata();
    $bubbleable_metadata->addCacheContexts($this->rendererConfig['required_cache_contexts']);

    // Determine the title: use the title provided by the main content if any,
    // otherwise get it from the routing information.
    $title = isset($main_content['#title']) ? $main_content['#title'] : $this->titleResolver->getTitle($request, $route_match->getRouteObject());
    // Ensure the title is rendered.
    if (is_array($title)) {
      $title = $this->renderer->renderRoot($title);
    }

    // Get drupal messages.
    $messages = $this->getMessages();

    // Get breadcrumbs.
    $breadcrumbs = $this->getBreadcrumbs($bubbleable_metadata);

    // Get metatags.
    $metatags = $this->getMetatags($route_match);

    $data = [
      'title' => $title,
      'messages' => $messages,
      'breadcrumbs' => $breadcrumbs['data'],
      'breadcrumbs_html' => $breadcrumbs['markup'],
      'metatags' => $metatags,
      'content' => $content ?: FALSE,
      'page' => 'layout--default',
    ];

    $this->moduleHandler->alter('lupus_ce_renderer_response', $data, $bubbleable_metadata, $request, $route_match);

    $response = new CacheableJsonResponse();
    $response->setData($data);
    $response->addCacheableDependency($bubbleable_metadata);
    return $response;
  }

  /**
   * Get drupal messages.
   *
   * @return array
   */
  private function getMessages() {
    $messages = $this->messenger->all();
    if (!empty($messages)) {
      $this->killSwitch->trigger();
    }
    if (isset($messages['status'])) {
      $success_messages = $messages['status'];
      unset($messages['status']);
      $messages['success'] = $success_messages;
    }
    $this->messenger->deleteAll();
    return $messages;
  }

  /**
   * Get breadcrumbs data and markup.
   *
   * @param \Drupal\Core\Render\BubbleableMetadata $bubbleable_metadata
   *   The bubbleable metatada.
   *
   * @return array
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  private function getBreadcrumbs(BubbleableMetadata &$bubbleable_metadata) {
    $breadcrumbs = [];
    $breadcrumb_block = $this->blockManager->createInstance('system_breadcrumb_block', []);
    $breadcrumb_render = $breadcrumb_block->build();
    if (!empty($breadcrumb_render['#links'])) {
      /** @var \Drupal\Core\Link $crumb */
      foreach ($breadcrumb_render['#links'] as $crumb) {
        $breadcrumbs[] = [
          'frontpage' => $crumb->getUrl()
            ->getRouteName() == '<front>' ? TRUE : FALSE,
          'url' => $crumb->getUrl()->toString(),
          'label' => (string) $crumb->getText(),
        ];
      }
      $bubbleable_metadata = BubbleableMetadata::createFromRenderArray($breadcrumb_render)
        ->merge($bubbleable_metadata);
    }

    return [
      'data' => $breadcrumbs,
      'markup' => $this->renderer->renderRoot($breadcrumb_render),
    ];
  }

  /**
   * Get metatags.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match, for context.
   *
   * @return array
   */
  private function getMetatags(RouteMatchInterface $route_match) {
    $metatags = metatag_generate_entity_metatags($route_match->getParameter('node'));
    $prepared_metatags = [
      'meta' => [],
      'link' => [],
    ];
    $blacklisted_metatags = $this->settings->get('blacklisted_metatags') ?? [];
    foreach ($metatags as $metatag_values) {
      $blacklisted = FALSE;
      if (isset($blacklisted_metatags[$metatag_values['#tag']])) {
        foreach ($metatag_values['#attributes'] as $value) {
          if (in_array($value, $blacklisted_metatags[$metatag_values['#tag']])) {
            $blacklisted = TRUE;
          }
        }
      }
      if (!$blacklisted) {
        $prepared_metatags[$metatag_values['#tag']][] = $metatag_values['#attributes'];
      }
    }
    return $prepared_metatags;
  }

}
