<?php

namespace Drupal\lupus_ce_renderer\EventSubscriber;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Cache\CacheableResponseInterface;
use Drupal\Core\EventSubscriber\MainContentViewSubscriber;
use Drupal\Core\Site\Settings;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Auto-enables the custom elements format for given base URLs.
 */
class CustomElementsFormatSubscriber implements EventSubscriberInterface {

  /**
   * Responds to kernel requests.
   *
   * @param \Symfony\Component\HttpKernel\Event\GetResponseEvent $event
   *   The event.
   */
  public function onKernelRequest(GetResponseEvent $event) {
    $request = $event->getRequest();
    if (Settings::get('lupus_ce_renderer_enable', TRUE) && !$request->query->get(MainContentViewSubscriber::WRAPPER_FORMAT)) {
      // Note that MainContentViewSubscriber takes care of caching responses by
      // wrapper format, sow e do not have to take care of that.
      $request->query->set(MainContentViewSubscriber::WRAPPER_FORMAT, 'custom_elements');
    }
  }

  /**
   * Responds to kernel responses.
   *
   * @param \Symfony\Component\HttpKernel\Event\FilterResponseEvent $event
   *   The event.
   */
  public function onKernelResponse(FilterResponseEvent $event) {
    $response = $event->getResponse();
    $request = $event->getRequest();

    // Remove the query parameter again, so it does not influence generated
    // cache keys. This is necessary for caches to work, since we enabled it
    // after initially caches are checked without the request parameter being in
    // the checked cache key. We cannot enable it earlier since it should not
    // be there when routes are matched.
    if (Settings::get('lupus_ce_renderer_enable', TRUE) && $request->query->get(MainContentViewSubscriber::WRAPPER_FORMAT) == 'custom_elements') {
      $request->query->remove(MainContentViewSubscriber::WRAPPER_FORMAT);
    }

    // Ensure we have different cache context when enabled per site via
    // settings.php, since the set query parameter is not part of the cache key.
    if ($response instanceof CacheableResponseInterface) {
      $response->addCacheableDependency((new CacheableMetadata())->addCacheContexts(['url.site']));
    }

    // Ensure redirects point to the given base URL if any.
    $base_url = Settings::get('lupus_ce_renderer_redirect_base_url');
    if ($base_url && $response instanceof RedirectResponse && strpos($response->getTargetUrl(), $base_url) !== 0) {
      $target = $response->getTargetUrl();
      $request = $event->getRequest();
      $new_url = str_replace($request->getSchemeAndHttpHost(), $base_url, $target);
      if ($new_url != $target) {
        $response->setTargetUrl($new_url);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      // Run after dynamic page cache (prio 27) and routing (prio 23).
      KernelEvents::REQUEST => ['onKernelRequest', 0],
      // Run before dynamic page cache which has prio 100.
      KernelEvents::RESPONSE => ['onKernelResponse', 500],
    ];
  }

}
